import java.util.HashMap;
import java.util.Map;


public aspect HandleAuthorization
{
	private static Person currentUser;
	private static Map<String,Person> personsMap = new HashMap<String,Person>();
	
	public static boolean putPerson(String key, Person value) {
		if(key == null || value == null || personsMap.get(key) != null) {
			return false;
		}
		personsMap.put(key, value);
		return true;
	}
	
	public static Person getCurrentUser()
	{
		return currentUser;
	}

	public static void logoutUser()
	{
		currentUser = null;
	}
	
	pointcut performingTransaction() :
		call(void ReserveRoomHandler.*(..))
		|| call(void CheckInHandler.*(..))
		|| call(void CheckOutHandler.*(..));
	
	void around() : performingTransaction()
	{
		if(isAuthorized()) {
			proceed();
		}
		else {
			GUI.setErrorMessage("Transaction not authorized (Aspect)");
		}
	}
	
	boolean isAuthorized()
	{
		boolean isAuthorized = false;
		
		if(currentUser == null) {
			System.out.println("Authorization failed: No user logged in!");
		} else if(currentUser.getSecurityLevel() > 5){
			System.out.println("Authorization failed: User does not have permission to perform action!");
		} else {
			isAuthorized = true;
		}
		return isAuthorized;
	}
	
	static boolean loginUser(String username, String password)
	{
		Person person = personsMap.get(username);
		if(person == null) {
			System.out.println("Incorrect username");
		} else if (!person.checkPass(password)) {
			System.out.println("Incorrent password");
		} else {
			currentUser = person; 
			return true;
		}
			
		return false;
	}
}
