
public aspect ReserveRoom {
	
	
	/** Defining which variables should be available for any advice using this pointcut
	 *  Here namely re, room & customer is made available. Note that we use "execution",
	 *  so that we can bind the newly created Reservation object to re with: this(re);
	 */
	pointcut createdReservation(Reservation re, Room room, Person customer) :
		execution(Reservation.new(Room, Person))
		// Here we are binding formals, using namebinding we define re to be the
		// target object, and room, customer to be the arguments given to the call
		&& this(re)
		&& args(room, customer);
	
	after(Reservation re, Room room, Person customer) returning: createdReservation(re, room, customer) && execution(Reservation.new(..)) {
		room.reserve();
		System.out.println("Reserved the room using an advice");
	}
	
	
	/*
	 * Examples
	 */
	before(Reservation re, Room room, Person customer) : createdReservation(re, room, customer) {
		System.out.println("Before pointcut createdReservation()");
	}
	
	pointcut constructorReservation() :
		call(Reservation.new(..));
	
	before() : constructorReservation() {
		System.out.println("before() : constructorReservation()");
	}
		
		
}
